
/* arithmetic.c	87/6/23 */

#include "dsm.h"

/* numeric */
double max(a, b) double a, b;
{
	return((a >= b) ? a : b);
}
double dabs(a) double a;
{
	return((a >= 0.0) ? a : -a);
}
double round(a) double a;
{
	return((a >= 0.0) ? (int)(a+0.5) : (int)(a-0.5));
}
iswhere(a, b, c) double a, b, c;
{
/*                                    */
/* return by a:  -1       0       1   */
/*               -------- bc -------> */
/*                                    */
/* return by a:  -2  -1   0   1   2   */
/*               ---- b ----- c ----> */
/*                                    */
#define	EPS	1.0e-5

	if (b > c-EPS) {
		if (a < b-EPS)
			return(-1);
		else if (a < c+EPS)
			return(0);
		else
			return(1);
	}
	else {
		if (a < b-EPS)
			return(-2);
		else if (a < b+EPS)
			return(-1);
		else if (a < c-EPS)
			return(0);
		else if (a < c+EPS)
			return(1);
		else
			return(2);
	}
}

/* vector */
v_len(x, y, z, rv) double x, y, z; register double *rv;
{
	*rv = sqrt(x*x+y*y+z*z);
}
v_norm(x, y, z, rx, ry, rz) double x, y, z; register double *rx, *ry, *rz;
{
	double	tp_dbl;

	v_len(x, y, z, &tp_dbl);
	if (isnonzero(tp_dbl)) {
		*rx = x/tp_dbl;
		*ry = y/tp_dbl;
		*rz = z/tp_dbl;
	}
	else
		*rx = *ry = *rz = 0.0;
}
v_sca(x, y, z, x2, y2, z2, rv) double x, y, z, x2, y2, z2; register double *rv;
{
	*rv = x*x2+y*y2+z*z2;
}
v_vec(x, y, z, x2, y2, z2, rx, ry, rz)
	double		x, y, z, x2, y2, z2;
	register double	*rx, *ry, *rz;
{
	*rx = y*z2-z*y2;
	*ry = z*x2-x*z2;
	*rz = x*y2-y*x2;
}
v_northo(x, y, z, x2, y2, z2) double x, y, z, x2, y2, z2;
{
	double	tp_dbl;

	v_sca(x, y, z, x2, y2, z2, &tp_dbl);
	return(isnonzero(tp_dbl));
}
v_ort(x, y, z, length, rx, ry, rz)
	double		x, y, z, length;
	register double	*rx, *ry, *rz;
{
	if (isnonzero(*ry = z)) {
		*rz = -y;
		*rx = 0.0;
	}
	else if (isnonzero(*rz = x)) {
		*rx = -z;
		*ry = 0.0;
	}
	else if (isnonzero(*rx = y)) {
		*ry = -x;
		*rz = 0.0;
	}
	else
		*rx = *ry = *rz = 0.0;
	v_norm(*rx, *ry, *rz, rx, ry, rz);
	*rx *= length;
	*ry *= length;
	*rz *= length;
}
v_foot(bx, by, bz, ex, ey, ez, px, py, pz, rx, ry, rz)
	double		bx, by, bz, ex, ey, ez, px, py, pz;
	register double	*rx, *ry, *rz;
{
/*                               */
/*          (px,py,pz)           */
/*               +               */
/*               |               */
/*               |--+            */
/* (bx,by,bz)    V  | (ex,ey,ez) */
/*  +------------*------------+  */
/*          (rx,ry,rz)           */
/*                               */
	double		edirx, ediry, edirz, t;

	edirx = ex-bx;
	ediry = ey-by;
	edirz = ez-bz;
	v_sca(edirx, ediry, edirz, px-bx, py-by, pz-bz, &t);
	t /= edirx*edirx+ediry*ediry+edirz*edirz;
	*rx = bx+t*edirx;
	*ry = by+t*ediry;
	*rz = bz+t*edirz;
}
v_deg(bx, by, bz, bfx, bfy, bfz, efx, efy, efz, ex, ey, ez, vx, vy, vz, deg)
	double		bx, by, bz, bfx, bfy, bfz;
	double		efx, efy, efz, ex, ey, ez;
	double		vx, vy, vz;
	register double	*deg;
{
/*                                 */
/*                 ^ (ex,ey,ez)    */
/*               B/                */
/* (vx,vy,vz)    /-+ (efx,efy,efz) */
/*  <-----------* deg              */
/*              ^--+ (bfx,bfy,bfz) */
/*             A|                  */
/*              |  (bx,by,bz)      */
/*                                 */
#define PI		3.1415926535897932
	double		Ax, Ay, Az, Bx, By, Bz;
	double		tp_sin, tp_cos;

	Ax = bx-bfx;
	Ay = by-bfy;
	Az = bz-bfz;
	Bx = ex-efx;
	By = ey-efy;
	Bz = ez-efz;
	v_norm(Ax, Ay, Az, &Ax, &Ay, &Az);
	v_norm(Bx, By, Bz, &Bx, &By, &Bz);
	v_norm(vx, vy, vz, &vx, &vy, &vz);
	v_sca(Ax, Ay, Az, Bx, By, Bz, &tp_cos);
	v_vec(Ax, Ay, Az, Bx, By, Bz, &Bx, &By, &Bz);
	v_sca(Bx, By, Bz, vx, vy, vz, &tp_sin);
	if (tp_cos > 1.0)
		tp_cos = 1.0;
	else if (tp_cos < -1.0)
		tp_cos = -1.0;
	*deg = acos(tp_cos)*180.0/PI;
	if (iswhere(tp_sin, 0.0, 0.0) < 0)
		*deg = 360.0-*deg;
}

/* motion */
m_papply(mot, x, y, z, rx, ry, rz)
	register struct motion	*mot;
	double			x, y, z;
	register double		*rx, *ry, *rz;
{
	*rx = mot->u1*x+mot->v1*y+mot->w1*z+mot->p1;
	*ry = mot->u2*x+mot->v2*y+mot->w2*z+mot->p2;
	*rz = mot->u3*x+mot->v3*y+mot->w3*z+mot->p3;
}
m_vapply(mot, x, y, z, rx, ry, rz)
	register struct motion	*mot;
	double			x, y, z;
	register double		*rx, *ry, *rz;
{
	*rx = mot->u1*x+mot->v1*y+mot->w1*z;
	*ry = mot->u2*x+mot->v2*y+mot->w2*z;
	*rz = mot->u3*x+mot->v3*y+mot->w3*z;
	v_norm(*rx, *ry, *rz, rx, ry, rz);
}
m_pmodif(x, y, z, mot, rmot)
	double			x, y, z;
	register struct motion	*mot, *rmot;
{
	rmot->u1 = mot->u1*x;
	rmot->u2 = mot->u2*x;
	rmot->u3 = mot->u3*x;
	rmot->v1 = mot->v1*y;
	rmot->v2 = mot->v2*y;
	rmot->v3 = mot->v3*y;
	rmot->w1 = mot->w1*z;
	rmot->w2 = mot->w2*z;
	rmot->w3 = mot->w3*z;
	rmot->p1 = mot->p1;
	rmot->p2 = mot->p2;
	rmot->p3 = mot->p3;
}
m_vmodif(x, y, z, mot, rmot)
	double			x, y, z;
	register struct motion	*mot, *rmot;
{
	rmot->u1 = mot->u1/x;
	rmot->u2 = mot->u2/x;
	rmot->u3 = mot->u3/x;
	rmot->v1 = mot->v1/y;
	rmot->v2 = mot->v2/y;
	rmot->v3 = mot->v3/y;
	rmot->w1 = mot->w1/z;
	rmot->w2 = mot->w2/z;
	rmot->w3 = mot->w3/z;
	rmot->p1 = mot->p1;
	rmot->p2 = mot->p2;
	rmot->p3 = mot->p3;
}
m_copy(mot, rmot) register struct motion *mot, *rmot;
{
	*rmot = *mot;
}
m_comp(mot, mot2, rmot) register struct motion *mot, *mot2, *rmot;
{
	struct motion	tp_mot;

	tp_mot.u1 = mot->u1*mot2->u1+mot->v1*mot2->u2+mot->w1*mot2->u3;
	tp_mot.u2 = mot->u2*mot2->u1+mot->v2*mot2->u2+mot->w2*mot2->u3;
	tp_mot.u3 = mot->u3*mot2->u1+mot->v3*mot2->u2+mot->w3*mot2->u3;
	tp_mot.v1 = mot->u1*mot2->v1+mot->v1*mot2->v2+mot->w1*mot2->v3;
	tp_mot.v2 = mot->u2*mot2->v1+mot->v2*mot2->v2+mot->w2*mot2->v3;
	tp_mot.v3 = mot->u3*mot2->v1+mot->v3*mot2->v2+mot->w3*mot2->v3;
	tp_mot.w1 = mot->u1*mot2->w1+mot->v1*mot2->w2+mot->w1*mot2->w3;
	tp_mot.w2 = mot->u2*mot2->w1+mot->v2*mot2->w2+mot->w2*mot2->w3;
	tp_mot.w3 = mot->u3*mot2->w1+mot->v3*mot2->w2+mot->w3*mot2->w3;
	tp_mot.p1 = mot->u1*mot2->p1+mot->v1*mot2->p2+mot->w1*mot2->p3+mot->p1;
	tp_mot.p2 = mot->u2*mot2->p1+mot->v2*mot2->p2+mot->w2*mot2->p3+mot->p2;
	tp_mot.p3 = mot->u3*mot2->p1+mot->v3*mot2->p2+mot->w3*mot2->p3+mot->p3;

	*rmot = tp_mot;
}
m_inv(mot, rmot) register struct motion *mot, *rmot;
{
	struct motion	tp_mot;

	tp_mot.u1 = mot->u1;
	tp_mot.u2 = mot->v1;
	tp_mot.u3 = mot->w1;
	tp_mot.v1 = mot->u2;
	tp_mot.v2 = mot->v2;
	tp_mot.v3 = mot->w2;
	tp_mot.w1 = mot->u3;
	tp_mot.w2 = mot->v3;
	tp_mot.w3 = mot->w3;
	tp_mot.p1 = -(mot->u1*mot->p1+mot->u2*mot->p2+mot->u3*mot->p3);
	tp_mot.p2 = -(mot->v1*mot->p1+mot->v2*mot->p2+mot->v3*mot->p3);
	tp_mot.p3 = -(mot->w1*mot->p1+mot->w2*mot->p2+mot->w3*mot->p3);

	*rmot = tp_mot;
}
m_movx(x, mot, rmot) double x; register struct motion *mot, *rmot;
{
	*rmot = *mot;
	rmot->p1 += x;
}
m_movy(y, mot, rmot) double y; register struct motion *mot, *rmot;
{
	*rmot = *mot;
	rmot->p2 += y;
}
m_movz(z, mot, rmot) double z; register struct motion *mot, *rmot;
{
	*rmot = *mot;
	rmot->p3 += z;
}
m_degx(x, mot, rmot) double x; register struct motion *mot, *rmot;
{
	m_rotx(x*PI/180.0, mot, rmot);
}
m_degy(y, mot, rmot) double y; register struct motion *mot, *rmot;
{
	m_roty(y*PI/180.0, mot, rmot);
}
m_degz(z, mot, rmot) double z; register struct motion *mot, *rmot;
{
	m_rotz(z*PI/180.0, mot, rmot);
}
m_rotx(x, mot, rmot) double x; register struct motion *mot, *rmot;
{
	struct motion	tp_mot;
	double		tp_cos, tp_sin;

	tp_cos = cos(x);
	tp_sin = sin(x);

	tp_mot.u1 = mot->u1;
	tp_mot.u2 = tp_cos*mot->u2-tp_sin*mot->u3;
	tp_mot.u3 = tp_sin*mot->u2+tp_cos*mot->u3;
	tp_mot.v1 = mot->v1;
	tp_mot.v2 = tp_cos*mot->v2-tp_sin*mot->v3;
	tp_mot.v3 = tp_sin*mot->v2+tp_cos*mot->v3;
	tp_mot.w1 = mot->w1;
	tp_mot.w2 = tp_cos*mot->w2-tp_sin*mot->w3;
	tp_mot.w3 = tp_sin*mot->w2+tp_cos*mot->w3;
	tp_mot.p1 = mot->p1;
	tp_mot.p2 = tp_cos*mot->p2-tp_sin*mot->p3;
	tp_mot.p3 = tp_sin*mot->p2+tp_cos*mot->p3;

	*rmot = tp_mot;
}
m_roty(y, mot, rmot) double y; register struct motion *mot, *rmot;
{
	struct motion	tp_mot;
	double		tp_cos, tp_sin;

	tp_cos = cos(y);
	tp_sin = sin(y);

	tp_mot.u1 =  tp_cos*mot->u1+tp_sin*mot->u3;
	tp_mot.u2 =  mot->u2;
	tp_mot.u3 = -tp_sin*mot->u1+tp_cos*mot->u3;
	tp_mot.v1 =  tp_cos*mot->v1+tp_sin*mot->v3;
	tp_mot.v2 =  mot->v2;
	tp_mot.v3 = -tp_sin*mot->v1+tp_cos*mot->v3;
	tp_mot.w1 =  tp_cos*mot->w1+tp_sin*mot->w3;
	tp_mot.w2 =  mot->w2;
	tp_mot.w3 = -tp_sin*mot->w1+tp_cos*mot->w3;
	tp_mot.p1 =  tp_cos*mot->p1+tp_sin*mot->p3;
	tp_mot.p2 =  mot->p2;
	tp_mot.p3 = -tp_sin*mot->p1+tp_cos*mot->p3;

	*rmot = tp_mot;
}
m_rotz(z, mot, rmot) double z; register struct motion *mot, *rmot;
{
	struct motion	tp_mot;
	double		tp_cos, tp_sin;

	tp_cos = cos(z);
	tp_sin = sin(z);

	tp_mot.u1 = tp_cos*mot->u1-tp_sin*mot->u2;
	tp_mot.u2 = tp_sin*mot->u1+tp_cos*mot->u2;
	tp_mot.u3 = mot->u3;
	tp_mot.v1 = tp_cos*mot->v1-tp_sin*mot->v2;
	tp_mot.v2 = tp_sin*mot->v1+tp_cos*mot->v2;
	tp_mot.v3 = mot->v3;
	tp_mot.w1 = tp_cos*mot->w1-tp_sin*mot->w2;
	tp_mot.w2 = tp_sin*mot->w1+tp_cos*mot->w2;
	tp_mot.w3 = mot->w3;
	tp_mot.p1 = tp_cos*mot->p1-tp_sin*mot->p2;
	tp_mot.p2 = tp_sin*mot->p1+tp_cos*mot->p2;
	tp_mot.p3 = mot->p3;

	*rmot = tp_mot;
}

