
/*=======================================================================
=	dsm.h	87/9/21							=
=======================================================================*/
#include <sys/types.h>
#include <sys/param.h>
#include <sys/times.h>
#include <sys/stat.h>
#include <signal.h>
#include <setjmp.h>
#include <stdio.h>
#include <math.h>

char		*program_name;	/* dsm */
char		*obj_fname;	/* object solid_file (READ) */
char		*obj_pname;	/* object solid_file (WRITE) */
char		*obj_ename;	/* object environment_file (READ/WRITE) */
char		*cur_wd;	/* current working directory */
char		*cur_fname;	/* current solid_file */
int		rread_count;

/************************************************************************
*	tbl.h	87/6/15							*
************************************************************************/
/* user configration parameters */
#define		IDTBL_SIZE	3000
#define		FTBL_SIZE	100

#define		PRIM_SIZE	40
#define		PFAC_SIZE	4000
#define		PXLO_SIZE	4000
#define		PXED_SIZE	8000
#define		PDPO_SIZE	4000
#define		PDVE_SIZE	8000

/*******************************
*      common definitions      *
*******************************/
#define		yywrap()	(1)
#define		rehash(x)	((x+137) % IDTBL_SIZE+1)

int		y_errnum;
int		y_errline;

/* system id (_t) entry */
double		*TIME_T_ENT;

/* idtable.type */
#define		T_UNDEF		0
#define		T_SOLID		0x0001
#define		T_LIST		0x0002
#define		T_REAL		0x0004

/* idtable.op */
#define		O_UNDEF		0

#define		OM_SOLID	0x1000
#define		O_ASB		0x1001
#define		O_UN		0x1002
#define		O_INT		0x1003
#define		O_DIF		0x1004

#define		OM_REAL		0x2000
#define		O_EQ		0x2001
#define		O_NE		0x2002
#define		O_GE		0x2003
#define		O_GT		0x2004
#define		O_LE		0x2005
#define		O_LT		0x2006
#define		O_PLUS		0x2011
#define		O_MINUS		0x2012
#define		O_MULT		0x2013
#define		O_DIV		0x2014
#define		O_NEG		0x2015
#define		O_ABS		0x2016
#define		O_POW		0x2021
#define		O_SIN		0x2022
#define		O_COS		0x2023
#define		O_TAN		0x2024
#define		O_ASIN		0x2025
#define		O_ACOS		0x2026
#define		O_ATAN		0x2027
#define		O_SQRT		0x2028
#define		O_LOG		0x2029
#define		O_EXP		0x202a
#define		O_RND		0x2031
#define		O_CONST		0x2032

#define		OM_ETC		0x4000
#define		O_FILE		0x4001
#define		O_ITE		0x4002
#define		O_ITEBODY	0x4003
#define		O_NVP		0x4004
#define		O_PARAM		0x4005
#define		O_LIST		0x4006
#define		O_AT		0x4007
#define		O_CWITH		0x4008
#define		O_WRT		0x4009
#define		O_TO		0x400a

/* idtable.reserve */
#define		S_UNDEF		0
#define		S_X		0x0001
#define		S_Y		0x0002
#define		S_Z		0x0004
#define		S_COL		0x0008
#define		S_MOVX		0x0010
#define		S_MOVY		0x0020
#define		S_MOVZ		0x0040
#define		S_DEGX		0x0100
#define		S_DEGY		0x0200
#define		S_DEGZ		0x0400
#define		S_ROTX		0x1000
#define		S_ROTY		0x2000
#define		S_ROTZ		0x4000

/* idtable.wanted */
#define		W_NO		0
#define		W_YES		1

/* table informations */
int		cur_file_ent;
int		last_file_ent;
int		last_tmpid_ent;

struct idtable {double	real;
		char	*name;
		char	*f_name;
		int	type;
		int	op;
		int	reserved;
		int	wanted;
		int	f_ent;
		int	l_link;
		int	r_link;
		int	alias;
		int	a_link; } idtable[IDTBL_SIZE+1];

struct ftable {	char	*name;
		char	*cwd;
		int	st_time;
		int	st_ino;
		int	st_dev;
		int	id_ent;
		int	prim_ent; } ftable[FTBL_SIZE+1];

/* table informations for primitives */
int		lst_prim_ent;
int		lst_pfac_ent;
int		lst_pxlo_ent;
int		lst_pxed_ent;
int		lst_pdpo_ent;
int		lst_pdve_ent;

struct prim {	double	pmaxdpo;
		int	pfactop, pfacnum;
		int	pxlotop, pxlonum;
		int	pxedtop, pxednum;
		int	pdpotop, pdponum;
		int	pdvetop, pdvenum; } prim[PRIM_SIZE+1];

struct pface {	int	facolor;
		int	fasssw;
		int	favect;
		int	xlotop; } pface[PFAC_SIZE+1];	

struct pxloop { int	r_link;
		int	xedtop; } pxloop[PXLO_SIZE+1];	

struct pxedge {	int	link1;
		int	link2;
		int	fa1;
		int	fa2;
		int	dp1;
		int	dp2;
		int	dv1;
		int	dv2; } pxedge[PXED_SIZE+1];	

struct pdpoint {double	x, y, z;
		int	ref; } pdpoint[PDPO_SIZE+1];	

struct pdvect { double	x, y, z;
		int	ref; } pdvect[PDVE_SIZE+1];	


/************************************************************************
*	csg.h	87/6/18							*
************************************************************************/
/* user configration parameters */
#define		CSG_SIZE	500
#define		MOTION_SIZE	500

/*******************************
*      common definitions      *
*******************************/
int		c_errnum;

/* list type */
#define		L_UNDEF		0
#define		L_GENERIC	0x0001
#define		L_PRIMITIVE	0x0002
#define		L_MOTION	0x0004

/* return type */
#define		R_UNDEF		0
#define		R_CSG		0x0001
#define		R_PARAM		0x0002
#define		R_MOTION	0x0004
#define		R_REAL		0x0008

/* csg.type */
#define		C_UNDEF		0
#define		C_PRIMITIVE	0x0001
#define		C_ASB		0x0002
#define		C_UN		0x0004
#define		C_INT		0x0008
#define		C_DIF		0x0010

/* table informations */
int		last_csg_ent;
int		last_motion_ent;

struct csg {	double	x, y, z, col;
		int	type;
		int	l_link;
		int	r_link;
		int	p_link;
		int	prim_ent;
		int	brep_ent;
		int	mot_ent; } csg[CSG_SIZE+1];

struct motion {	double	u1, v1, w1, p1;
		double	u2, v2, w2, p2;
		double	u3, v3, w3, p3; } motion[MOTION_SIZE+1];

struct motion	*STD_MOTION;


/************************************************************************
*	brep.h	87/6/18							*
************************************************************************/
/* user configration parameters */
#define		BREP_SIZE	400
#define		FAC_SIZE	4000
#define		XLO_SIZE	4000
#define		XED_SIZE	8000
#define		DPO_SIZE	6000
#define		DVE_SIZE	8000
#define		PXX_SIZE	2000
#define		EXX_SIZE	2000

/*******************************
*      common definitions      *
*******************************/
/* table informations */
struct breps {	struct brep	*brep;
		struct face	*face;
		struct xloop	*xloop;
		struct xedge	*xedge;
		struct dpoint	*dpoint;
		struct dvect	*dvect;
		struct pxx	*pxx;
		struct exx	*exx;
		int		ls_brep_ent;
		int		ls_fac_ent;
		int		ls_xlo_ent;
		int		ls_xed_ent;
		int		ls_dpo_ent;
		int		ls_dve_ent;
		int		ls_pxx_ent;
		int		ls_exx_ent;
		int		fre_xlo_ent;
		int		fre_xed_ent;
		int		fre_dpo_ent;
		int		fre_dve_ent;
		int		pe; }
#ifdef DIADIC
					breps2,
#endif
					breps1;

struct brep {	double	dpocex, dpocey, dpocez, maxdpo;
		int	inview;
		int	factop, facnum;
		int	xlotop, xlonum;
		int	xedtop, xednum;
		int	dpotop, dponum;
		int	dvetop, dvenum; }
#ifdef DIADIC
					brep2[BREP_SIZE+1],
#endif
					brep1[BREP_SIZE+1];

struct face {	int	facolor;
		int	fasssw;
		int	favect;
		int	a_link;
		int	xlotop;
		int	exxtop;
		int	pe;
		int	work; }
#ifdef DIADIC
					face2[FAC_SIZE+1],
#endif
					face1[FAC_SIZE+1];

struct xloop {	int	bl_link;
		int	br_link;
		int	r_link;
		int	xedtop; }
#ifdef DIADIC
					xloop2[XLO_SIZE+1],
#endif
					xloop1[XLO_SIZE+1];

struct xedge {	double	io1;		/* 0-360:boundary        */
		double	io2; 		/*     0:outer 360:inner */
		int	bd1;		/* 1:on edge boundary */
		int	bd2;		/* 1:on edge boundary */
		int	bl_link;
		int	br_link;
		int	a_link;
		int	link1;
		int	link2;
		int	fa1;
		int	fa2;
		int	dp1;
		int	dp2;
		int	dv1;
		int	dv2; }
#ifdef DIADIC
					xedge2[XED_SIZE+1],
#endif
					xedge1[XED_SIZE+1];

struct dpoint {	double	x, y, z;
		int	ref;
		int	bl_link;
		int	br_link;
		int	a_link;
		int	pxxtop; }
#ifdef DIADIC
					dpoint2[DPO_SIZE+1],
#endif
					dpoint1[DPO_SIZE+1];

struct dvect {	double	x, y, z;
		int	ref;
		int	bl_link;
		int	br_link;
		int	a_link; }
#ifdef DIADIC
					dvect2[DVE_SIZE+1],
#endif
					dvect1[DVE_SIZE+1];

struct pxx {	int	r_link;
		int	fac;
		int	xed; }	
#ifdef DIADIC
					pxx2[PXX_SIZE+1],
#endif
					pxx1[PXX_SIZE+1];

struct exx {	int	r_link;
		int	d_link;
		int	xed; }
#ifdef DIADIC
					exx2[EXX_SIZE+1],
#endif
					exx1[EXX_SIZE+1];

/* for work */
struct dpowrk {	double	x, y, z; }	dpowrk[DPO_SIZE+1];	
struct dvewrk {	double	x, y, z; }	dvewrk[DVE_SIZE+1];	


/************************************************************************
*	arithmetic.h	87/6/18						*
************************************************************************/
/*******************************
*      common definitions      *
*******************************/
#define		isnonzero(x)		iswhere(x, 0.0, 0.0)
#define		isnonequ(x,y)		iswhere((x)-(y), 0.0, 0.0)
#define		isabswhere(x,y,z)	iswhere(dabs(x), y, z)

/************************************************************************
*	disp.h	87/7/14							*
************************************************************************/
/*******************************
*      common definitions      *
*******************************/
#define		SCR_LOC		(-1.0e-3)

